/*	Mobile Multicast Control Protocol Implementation.
 *
 *	mcs.c
 *	This source includes implementation of the Multicast Contents Server
 *
 *	Contact :	Jaesung Park (knucsid@gmail.com)
 *				Jiin Kim
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>

#define		TTL		64
#define		BUFSIZE	2048

void err_print( char *msg );


int main( int argc, char **argv )
{
	int send_sock;
	struct sockaddr_in multi_addr;
	int multi_TTL = TTL;
	int state;
	FILE *fp;
	char buf[BUFSIZE];

	ssize_t nbytes;

	if( argc != 4 ) {
		printf( "Usage : %s [ip] [port] [filename]\n", argv[0] );
		printf( "argc = %d\n", argc );
		exit(1);
	}

	send_sock = socket( PF_INET, SOCK_DGRAM, IPPROTO_UDP );
	if( send_sock == -1 )
		err_print( "socket() error");

	memset( &multi_addr, 0, sizeof(multi_addr) );
	multi_addr.sin_family = AF_INET;
	multi_addr.sin_addr.s_addr = inet_addr(argv[1]);
	multi_addr.sin_port = htons(atoi(argv[2]));

	state = setsockopt( send_sock, IPPROTO_IP, IP_MULTICAST_TTL,
						(void*) &multi_TTL, sizeof(multi_TTL) );

	if(state)
		err_print( "setsockopt() error" );

	if( (fp=fopen(argv[3], "r")) == NULL )
		err_print( "fopen() error" );

	sleep(5);
	while( !feof(fp) )
	{
		nbytes = fread( buf, 1, BUFSIZE, fp );
		sendto( send_sock, buf, nbytes, 0,
				(struct sockaddr*)&multi_addr, sizeof(multi_addr) );
		usleep(300);
	}

	fclose(fp);
	close(send_sock);
	return 0;
}

void err_print( char *msg )
{
	perror(msg);
	exit(1);
}
