
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <pthread.h>

#include <mmcp.h>

#define		BUFSIZE	2048

void *recv_data( void *arg );
void *player( void *arg );

void err_print( char *msg );

char buf[BUFSIZE];
FILE *fp;

int main( int argc, char **argv )
{
	int sock, msd;
	struct sockaddr_in local_addr;
	struct sockaddr_in sm_addr;
	struct sockaddr_in mcs_addr;
	int state;
	struct ip_mreq join_addr;

	int option;
	socklen_t optlen;

	pthread_t recv_thread, play_thread;
	void* t_return;

	if( argc != 2 ) {
		printf( "Usage : %s [SM ADDR]\n", argv[0] );
		exit(1);
	}

	msd = mmcp_socket( PF_INET, IPPROTO_UDP, MMCP_ROLE_MN );
	if( msd < 0 )
		err_print( "mmcp_socket() error:" );

	printf( "msd = %d\n", msd );

	sock = socket( PF_INET, SOCK_DGRAM, IPPROTO_UDP );
	if( sock == -1 )
		err_print( "socket() error:" );

	memset( &sm_addr, 0, sizeof(sm_addr) );
	sm_addr.sin_family = AF_INET;
	sm_addr.sin_addr.s_addr = inet_addr(argv[1]);
	sm_addr.sin_port = htons(MMCP_SM_PORT);

	mmcp_join( msd, sock,
				(struct sockaddr_storage *) &sm_addr, 
				(struct sockaddr_storage *) &mcs_addr );

	setsockopt( sock, SOL_SOCKET, SO_REUSEADDR, &option, sizeof(option) );

	memset( &local_addr, 0, sizeof(local_addr) );
	local_addr.sin_family = AF_INET;
	local_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	local_addr.sin_port = mcs_addr.sin_port;

	if( bind(sock, (struct sockaddr *) &local_addr, sizeof(local_addr)) < 0 )
		err_print( "bind() error" );

	join_addr.imr_multiaddr.s_addr = mcs_addr.sin_addr.s_addr;
	join_addr.imr_interface.s_addr = htonl(INADDR_ANY);

	state = setsockopt( sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
						(void*) &join_addr, sizeof(join_addr) );
	if( state )
		err_print( "setsockopt() error" );

	if( !(fp = fopen("result.ts", "w")) )
		err_print( "File Open() error" );

	pthread_create( &recv_thread, NULL, recv_data, (void*)sock );
	pthread_create( &play_thread, NULL, player, NULL );
	
	pthread_join( recv_thread, &t_return );
	pthread_join( play_thread, &t_return );

	close(sock);
	return 0;
}

void *recv_data( void *arg )
{
	int sk = (int) arg;
	ssize_t nbytes;
//	int i=0;

	printf( "read start!\n" );

	while(1) {
//		i++;
		nbytes = recvfrom( sk, buf, BUFSIZE, 0, NULL, NULL );
//		printf("nbytes = %d, i = %d\n", nbytes, i);
		if( nbytes <= 0 )
			break;

		buf[nbytes] = 0;

		nbytes = fwrite( buf, 1, nbytes, fp );
//		printf( "fwrite = %d\n", nbytes );
	}

	return 0;
}

void *player( void *arg )
{
	sleep(1);
	system( "mplayer -quiet result.ts 1>/dev/null 2>/dev/null -vf screenshot" );

	return 0;
}

void err_print( char *msg )
{
	perror(msg);
	exit(1);
}
