/*	Mobile Multicast Control Protocol Implementation.
 *
 *	This file is part of the MMCP over UDP Implementation.
 *
 *	mmcp.h
 *	This header defines the MMCP Internal Functions used to Protocol
 *	Operations.
 *
 *	Contact :	Jaesung Park(knucsid@gmail.com)
 *				Jiin Kim
 */


#ifndef	_MMCP_H
#define	_MMCP_H

/* ----------------------------------------------------------------- *
 *	Header Files Included
 * ----------------------------------------------------------------- */

#include <header/protocol.h>
#include <header/structs.h>
#include <header/types.h>


#ifdef	__cplusplus
extern "C" {
#endif

/* ----------------------------------------------------------------- *
 *	Define MMCP
 * ----------------------------------------------------------------- */

#define	MMCP_SM_PORT			8585
#define	MMCP_LMC_PORT_ONE		9926
#define MMCP_LMC_PORT_TWO		9939
#define	MMC_VER					0x1

#define	MMCP_CAPTURE_DEV		"eth1"
#define MMCP_USE_DEV			"rausb0"

#define MMCP_MN_ID				"JS.Park"
#define MMCP_POA_ONE			"CPL-AP1"
#define MMCP_POA_TWO			"CPL-AP2"



/* ----------------------------------------------------------------- *
 *	Event Invocation Functions
 * ----------------------------------------------------------------- */

/* control.c ------------------------------------------------------- */

extern int mmcp_ops_start_events( struct mmcp_socket *mso );
extern int mmcp_ops_end_events( struct mmcp_socket *mso );
extern int mmcp_ops_maintain_events( struct mmcp_socket *mso );


/* ----------------------------------------------------------------- *
 *	State Machine Control Functions
 * ----------------------------------------------------------------- */

/* control.c ------------------------------------------------------- */

extern int mmcp_ops_start_join( struct mmcp_socket *mso );

extern int mmcp_ops_user_leave( struct mmcp_socket *mso );

extern int mmcp_ops_accept_session_join( 
		struct mmcp_socket *mso, mmcp_role_t role, mmcp_addr_t *from );

extern int mmcp_ops_deny_session_join( 
		struct mmcp_socket *mso, mmcp_role_t role, mmcp_addr_t *from );

extern int mmcp_ops_accept_local_join( 
		struct mmcp_socket *mso, mmcp_addr_t *from );

extern int mmcp_ops_deny_local_join( 
		struct mmcp_socket *mso, mmcp_addr_t *from );

extern int mmcp_ops_request_handover( struct mmcp_socket *mso );

extern int mmcp_ops_transfer_handover( struct mmcp_socket *mso );

extern int mmcp_ops_transfer_ack_handover(
		struct mmcp_socket *mso, mmcp_addr_t *from );

extern int mmcp_ops_local_join( struct mmcp_socket *mso );

extern int mmcp_ops_igmp_send( struct mmcp_socket *mso );

extern int mmcp_ops_send_status_report( struct mmcp_socket *mso );

extern int mmcp_ops_send_astatus_report( struct mmcp_socket *mso );

extern int mmcp_ops_send_status_probe( struct mmcp_socket *mso );


/* ----------------------------------------------------------------- *
 *	Event/Timer Control Functions
 * ----------------------------------------------------------------- */

/* event.c --------------------------------------------------------- */

extern void mmcp_event_dispatch( struct mmcp_socket *mso, fd_set *fds );

/* timer.c --------------------------------------------------------- */

extern void mmcp_timer_dispatch( struct mmcp_socket *mso );


/* ----------------------------------------------------------------- *
 *	Get Interface Information Functions
 * ----------------------------------------------------------------- */

/* control.c ------------------------------------------------------- */

extern mn_status get_inter_io( char *_name );


/* ----------------------------------------------------------------- *
 *	Packet Create Functions
 * ----------------------------------------------------------------- */

/* control.c ------------------------------------------------------- */

extern mmcp_pkt_sjr *mmcp_create_sjr( struct mmcp_socket *mso );
extern mmcp_pkt_sjc *mmcp_create_sjc( struct mmcp_socket *mso, int f );

extern mmcp_pkt_ljr *mmcp_create_ljr( struct mmcp_socket *mso );
extern mmcp_pkt_ljc *mmcp_create_ljc( struct mmcp_socket *mso, int f );

extern mmcp_pkt_ulr *mmcp_create_ulr( struct mmcp_socket *mso );
extern mmcp_pkt_ulc *mmcp_create_ulc( struct mmcp_socket *mso );

extern mmcp_pkt_usr *mmcp_create_usr( struct mmcp_socket *mso );
extern mmcp_pkt_asr *mmcp_create_asr( struct mmcp_socket *mso );
extern mmcp_pkt_usp *mmcp_create_usp( struct mmcp_socket *mso );

extern mmcp_pkt_hir *mmcp_create_hir( struct mmcp_socket *mso );
extern mmcp_pkt_hct *mmcp_create_hct( struct mmcp_socket *mso );
extern mmcp_pkt_hta *mmcp_create_hta( struct mmcp_socket *mso );
extern mmcp_pkt_hic *mmcp_create_hic( struct mmcp_socket *mso, int f );

void *mmcp_pkt_alloc( mmcp_packet_t type, int length, int domain );

static inline ssize_t mmcp_pkt_transmit( int sd, void *pkt, size_t pktlen,
										mmcp_addr_t *to, socklen_t tolen )
{
#ifdef	_MMCP_DEBUG
	char str[SA_IN6_LEN];
	printf( "\nmmcp_pkt_transmit ----------------------------------\n" );
	printf( "	to : %s\n", inet_ntop( AF_INET,
				&((SA_IN *) to)->sin_addr, str, tolen) );
	printf( "---------------------------------- mmcp_pkt_transmit\n\n" );
#endif

	return sendto( sd, (struct mmcp_basehdr *) pkt, pktlen, 0, 
					(SA *) to, tolen );
}

static inline ssize_t mmcp_skb_transmit( struct mmcp_skbuff *skb )
{
	skb->has_sent = 1;

	return sendto( skb->sd, (struct mmcp_basehdr *) skb->pkt, skb->pktlen, 0,
					(SA *) &skb->addr, skb->addrlen );
}


#ifdef	__cplusplus
}
#endif

#endif	// _MMCP_TYPES_H
