/*	Mobile Multicast Control Protocol Implementation.
 *
 *	This file is part of the MMCP over UDP Implementation.
 *
 *	protocol.h
 *	This header represents the MMCP protocol specification.
 *
 *	Contact :	Jaesung Park (knucsid@gmail.com)
 *				Jiin Kim
 */

#include <header/types.h>

#ifndef	_MMCP_PROTOCOL_H
#define	_MMCP_PROTOCOL_H

#ifdef	__cplusplus
extern "C" {
#endif

/*	6.4 MMCP Messages */
typedef enum
{
	MMCP_PKT_SJR	= 0x01,
#define MMCP_PKT_SJR	MMCP_PKT_SJR
	MMCP_PKT_SJC	= 0x02,
#define MMCP_PKT_SJC	MMCP_PKT_SJC
	MMCP_PKT_LJR	= 0x03,
#define MMCP_PKT_LJR	MMCP_PKT_LJR
	MMCP_PKT_LJC	= 0x04,
#define MMCP_PKT_LJC	MMCP_PKT_LJC
	MMCP_PKT_ULR	= 0x05,
#define MMCP_PKT_ULR	MMCP_PKT_ULR
	MMCP_PKT_ULC	= 0x06,
#define MMCP_PKT_ULC	MMCP_PKT_ULC
	MMCP_PKT_USR	= 0x07,
#define MMCP_PKT_USR	MMCP_PKT_USR
	MMCP_PKT_ASR	= 0x08,
#define MMCP_PKT_ASR	MMCP_PKT_ASR
	MMCP_PKT_USP	= 0x09,
#define MMCP_PKT_USP	MMCP_PKT_USP
	MMCP_PKT_HIR	= 0x0A,
#define MMCP_PKT_HIR	MMCP_PKT_HIR
	MMCP_PKT_HCT	= 0x0C,
#define MMCP_PKT_HCT	MMCP_PKT_HCT
	MMCP_PKT_HTA	= 0x0D,
#define MMCP_PKT_HTA	MMCP_PKT_HTA
	MMCP_PKT_HIC	= 0x0E
#define MMCP_PKT_HIC	MMCP_PKT_HIC
} mmcp_packet_t;

//	8.1 Base Header
struct mmcp_basehdr
{
	__u8	pkt_type;			/* Packet Type */
	__u8	mn_leng:4,
			reserved:3,			/* Reserved */
			f:1;				/* F Flag Bit */
	__u16	payload_leng;		/* Payload Length */
	__u32	session_id;			/* Session ID */
};

#define	MMCP_BASEHDR_LEN		sizeof(struct mmcp_basehdr)


//	8.2.1 Session Join Request (SJR)
typedef struct {
	struct mmcp_basehdr		basehdr;
	__s8	mn_id[MMCP_MNID_LEN];
	__s8	poa_id[MMCP_POAID_LEN];
} mmcp_pkt_sjr;

#define	MMCP_PKT_SJR_LEN		sizeof(mmcp_pkt_sjr)


//	8.2.2 Sessio Join Confirm (SJC)
typedef struct {
	struct mmcp_basehdr		basehdr;
	__u16	group_port; 
	__u16	lmc_port;
	mmcp_ip_t group_ip;
	mmcp_ip_t lmc_ip;
} mmcp_pkt_sjc;

#define	MMCP_PKT_SJC_LEN		sizeof(mmcp_pkt_sjc)


//	8.2.3 Local Join Request (LJR)
typedef struct {
	struct mmcp_basehdr		basehdr;
	__s8	mn_id[MMCP_MNID_LEN];
} mmcp_pkt_ljr;

#define	MMCP_PKT_LJR_LEN		sizeof(mmcp_pkt_ljr)


//	8.2.4 Local Join Confirm (LJC)
typedef struct {
	struct mmcp_basehdr		basehdr;
} mmcp_pkt_ljc;

#define	MMCP_PKT_LJC_LEN		sizeof(mmcp_pkt_ljc)


//	8.3.1 User Leave Request (ULR)
typedef struct {
	struct mmcp_basehdr		basehdr;
} mmcp_pkt_ulr;

#define	MMCP_PKT_ULR_LEN		sizeof(mmcp_pkt_ulr)


//	8.3.2 User Leave Confirm (ULC)
typedef struct {
	struct mmcp_basehdr		basehdr;
} mmcp_pkt_ulc;

#define	MMCP_PKT_ULC_LEN		sizeof(mmcp_pkt_ulc)


//	8.4.1 User Status Report (USR)
typedef struct {
	struct mmcp_basehdr		basehdr;
	__u32					rcv_byte;
	__u32					elapsed_time;
	__s8					mn_id[MMCP_MNID_LEN];
} mmcp_pkt_usr;

#define	MMCP_PKT_USR_LEN		sizeof(mmcp_pkt_usr)


//	8.4.2 Aggregate Status Report (ASR)
typedef struct {
	struct mmcp_basehdr		basehdr;
	__u16					number_of_mn;
	__u16					reserved;
	__s8					poa_id[MMCP_POAID_LEN];
	mn_status				mmcp_mn_info[4];
} mmcp_pkt_asr;

#define	MMCP_PKT_ASR_LEN		sizeof(mmcp_pkt_asr)


//	8.4.4 User Status Probe (USP)
typedef struct {
	struct mmcp_basehdr		basehdr;
} mmcp_pkt_usp;

#define	MMCP_PKT_USP_LEN		sizeof(mmcp_pkt_usp)


//	8.5.1 Handover Initiation Request (HIR)
typedef struct {
	struct mmcp_basehdr		basehdr;
//	mmcp_ip_t				group_ip;
	__s8					mn_id[MMCP_MNID_LEN];
	__s8					poa_id[MMCP_POAID_LEN];
} mmcp_pkt_hir;

#define	MMCP_PKT_HIR_LEN		sizeof(mmcp_pkt_hir)


//	8.5.3 Handover Context Transfer (HCT)
typedef struct {
	struct mmcp_basehdr		basehdr;
	mmcp_ip_t				group_ip;
	__s8 					mn_id[MMCP_MNID_LEN];
} mmcp_pkt_hct;

#define	MMCP_PKT_HCT_LEN		sizeof(mmcp_pkt_hct)


//	8.5.4 Handover Transfer Acknowledgement (HTA)
typedef struct {
	struct mmcp_basehdr		basehdr;
} mmcp_pkt_hta;

#define	MMCP_PKT_HTA_LEN		sizeof(mmcp_pkt_hta)


//	8.5.5 Handover Initiation Confirm (HIC)
typedef struct {
	struct mmcp_basehdr		basehdr;
	__s8					mn_id[MMCP_MNID_LEN];
	__u16					newlmc_port;
	__u16					reserved;
	mmcp_ip_t				newlmc_ip;

} mmcp_pkt_hic;

#define	MMCP_PKT_HIC_LEN		sizeof(mmcp_pkt_hic)


#ifdef	__cplusplus
}
#endif

#endif	/* _MMCP_PROTOCOL_H */
