#include <header/ieee802_11.h>
#include <header/mmcp.h>

#include <sys/time.h>
#include <netinet/in.h>
#include <pcap.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <pthread.h>

#define	PROMISCUOUS			1

#define	TTEST2(var, l) (snapend-(l) <= snapend && \
		(const unsigned char *)&(var) <= snapend-(l))

int sockfd;
int chcnt;
int link_type;
const unsigned char *packetp;
const unsigned char *snapend;

void packet_analysis( unsigned char *user, const struct pcap_pkthdr *h,
						const unsigned char *p );

struct printer {
	pcap_handler f;
	int type;
};

/*
 *	Data-link Level type code.
 *
 *	DLT_NULL				0		no link-layer encapsulation
 *	DLT_EN10MB				1		Ethernet (10MB)
 *	DLT_IEEE802				6		IEEE 802 Networks
 *	DLT_IEEE802_11			105		IEEE 802.11 wireless
 *	DLT_PRISM_HEADER		119		For 802.11 cards using the Prism II chips
 *	DLT_AIRONET_HEADER		120 	Reserved for Aironet 802.11 cards
 *	DLT_IEEE802_11_RADIO	127		Header for 802.11 plus including radio
 *
 *	define pcap-bpf.h header file.
 *
 */

static struct printer printers[] = {
	{ packet_analysis, DLT_IEEE802_11 },
	{ packet_analysis, DLT_PRISM_HEADER },		/* USB Interface Card */
	{ packet_analysis, DLT_IEEE802_11_RADIO },	/* built-in Interface Card */
	{ NULL, 0 }
};

static pcap_handler lookup_printer( int type )
{
	struct printer *p;

	for( p = printers; p->f; ++p )
	{
		if( type == p->type ) {
			link_type = type;
			return p->f;
		}
	}

	perror( "unknown data link type" );

	return NULL;
}

static int handle_beacon( struct mmcp_socket *mso, unsigned short fd, 
		const struct mgmt_header_t *pmh, const unsigned char *p )
{
	struct mgmt_body_t pbody;
	int offset = 0;

	memset( &pbody, 0, sizeof(struct mgmt_body_t) );

	if( !TTEST2(*p, 12) )
		return -1;
	offset += 12;

	unsigned char temp=0;

	if( !TTEST2(*(p+offset), 1))
		return -1;

	temp = EXTRACT_LE_8BITS(p+offset);
	if( temp == E_SSID ) {

		if (!TTEST2(*(p+offset), 2))
			return -1;

		memcpy(&pbody.ssid,p+offset,2); 
		offset += 2;

		if (pbody.ssid.length > 0)
		{
			if (!TTEST2(*(p+offset), pbody.ssid.length))
				return -1;
			memcpy(pbody.ssid.ssid,p+offset,pbody.ssid.length); 
			offset += pbody.ssid.length;
			pbody.ssid.ssid[pbody.ssid.length]='\0';
		}

//		printf( "ssid: %s\n", pbody.ssid.ssid );
		if( !strncmp( pbody.ssid.ssid, "CPL-AP2", sizeof("CPL-AP2")) ) {
			system("./stop.sh");
//			printf( "goodgood\n" );

			if( mmcp_ops_request_handover(mso) < 0 ) {
				err_ret( "handover procedure: "
						 "mmcp_ops_request_handover() error", -1 );
			}

			system( "./hd.sh" );

			if( mmcp_ops_local_join(mso) < 0 ) {
				err_ret( "handover procedure: "
						 "mmcp_ops_local_join() error", -1 );
			}
		}
	}

	return 0;
}

void packet_analysis( unsigned char *user, const struct pcap_pkthdr *h,
						const unsigned char *p )
{
	struct mmcp_socket *mso = NULL;
	struct mmcp_mn *mn = NULL;
	unsigned int caplen = h->caplen;
	unsigned int length = h->len;
	unsigned int HEADER_LENGTH;
	
	unsigned short fc;
	struct timeval current_time;

	mso = (struct mmcp_socket *) user;
	mn = mso->ep->mn;
	chcnt = 0;

	memcpy( &current_time, &h->ts, sizeof(struct timeval) );

	if( caplen < IEEE802_11_FC_LEN ) {
		printf( "caplen < IEEE802_11_FC_LEN\n" );
		return ;
	}

	if( link_type == DLT_IEEE802_11_RADIO )
		p = p + 25;
	if( link_type == DLT_PRISM_HEADER )
		p = p + 144;
	fc = EXTRACT_LE_16BITS(p);

	packetp = p;
	snapend = p + caplen;

	if( FC_TYPE(fc) == T_MGMT ) {
        HEADER_LENGTH = MGMT_HEADER_LEN;
		length -= HEADER_LENGTH;
		caplen -= HEADER_LENGTH;
		p += HEADER_LENGTH;

		if( FC_SUBTYPE(fc) == ST_BEACON ) {
		//	printf( "ST_BEACON\n" );
			if( handle_beacon( mso, fc, 
					(const struct mgmt_header_t*)packetp, p ) == 1 ) {
				printf( "handle_beacon error\n" );
				return;
			}

		}
	}

//	printf( "packet_analysis\n" );
//	while( chcnt < h->len ) {
//		printf( "%02x ", *(p++) );
//		if( !(++chcnt % 16 ) )
//			printf("\n");
//	}
//	printf( "\n");

}

void wlan_ieee80211_capture( struct mmcp_socket *mso, char *device )
{
	pcap_t *pd;
	struct bpf_program		fcode;
	pcap_handler			printer;
	char 					ebuf[PCAP_ERRBUF_SIZE];
	int 					i, snaplen = 1514, packetcnt=0;
	bpf_u_int32 			localnet, netmask;
	unsigned char 			*pcap_userdata;
	char					*filter_rule = NULL;

//	sleep(2);
	system("./start.sh");
	if( device == NULL ) {
		if( (device = pcap_lookupdev(ebuf)) == NULL ) {
			perror(ebuf);
			exit(-1);
		}
	}

	fprintf( stdout, "device = %s\n", device );

	pd = pcap_open_live( device, snaplen, PROMISCUOUS, 1000, ebuf );
	if( pd == NULL ) {
		perror(ebuf);
		exit(-1);
	}

	i = pcap_snapshot(pd);
	if( snaplen < i ) {
		perror(ebuf);
		exit(-1);
	}

	if( pcap_lookupnet(device, &localnet, &netmask, ebuf) < 0 ) {
		perror(ebuf);
		exit(-1);
	}

	if( pcap_compile(pd, &fcode, filter_rule, 0, netmask) < 0 ) {
		perror(ebuf);
		exit(-1);
	}

	if( pcap_setfilter(pd, &fcode) < 0 ) {
		perror(ebuf);
		exit(-1);
	}

	printer = lookup_printer( pcap_datalink(pd) );
	pcap_userdata = (unsigned char *) mso;

	if( pcap_loop(pd, packetcnt, printer, pcap_userdata) < 0 ) {
		perror("pcap_loop error");
		exit(-1);
	}

	pcap_close(pd);

	return;

}
